#!/bin/csh 
#BSUB -a mvapich
#BSUB -J mvapich_48c
#BSUB -q test
#BSUB -n 48
#BSUB -W 1:00
#BSUB -o %J.out
#BSUB -e %J.err
#BSUB -R "span[ptile=48]"
#BSUB -R "ipathavail==0"

## Export all my environment variables to the job
source /pkg/local/Modules/3.2.10/init/csh
setenv MODULEPATH "/pkg/local/Modules/modulefiles:/home/j07hsu00/opt/Modules/modulefiles"
module purge
module load compiler/intel/2015 mpi/intel/2015/mvapich2/2.1 netcdf/4.4.0/intel-2015_hdf5-1.8.16 pnetcdf/1.7.0/intel-2015_mvapich2-2.1
module list

limit coredumpsize 1000000
limit datasize  unlimited
limit stacksize unlimited

mpirun ./pre.exe > prep.out

