
#source /usr/share/Modules/init/csh

#module purge
module load intel-13.1/compiler intel-13.1/openmpi-1.8.8 intel-13.1/netcdf-4.4.1-nc4
module list
export PNETCDF=/cluster/intel-13.1/pnetcdf-1.3.1-openmpi

