module const
    integer, parameter::    i2t=320
    integer, parameter::    j2t=288
    integer, parameter::    k0=41

    integer, parameter::    i0t=i2t+2
    integer, parameter::    j0t=j2t+2
    integer, parameter::    i1t=i0t-1
    integer, parameter::    j1t=j0t-1 

    integer, parameter::    ngx0=4
    integer, parameter::    ngy0=6
    integer, parameter::    n0=4
    integer, parameter::    ng0=ngx0*ngy0 
    integer, parameter::    ngx1=ngx0-1
    integer, parameter::    ngy1=ngy0-1
    integer, parameter::    nb0=j2t/ngy0/n0
    integer, parameter::    nb1=nb0-1 
    integer, parameter::    i0=i2t/ngx0+2
    integer, parameter::    i1=i0-1
    integer, parameter::    i2=i0-2 
    integer, parameter::    j0=j2t/ngy0+2
    integer, parameter::    j1=j0-1
    integer, parameter::    j2=j0-2 
    integer, parameter::    k1=k0-1
    integer, parameter::    k2=k0-2

    integer, parameter::    ncopt=30
    integer, parameter::    ini_clim=1

    integer, parameter::   nbg0=nb0*ngy0
    integer, parameter::   nbg1=nbg0-1
    integer, parameter::   i01=i0+1
    integer, parameter::   ibir=i2t/ngy0
    integer, parameter::   jg0=j2*ngy0+2
    integer, parameter::   j01=j0+1
    integer, parameter::   jg1=jg0-1
    integer, parameter::   jg2=jg0-2
    integer, parameter::   ig0=i2*ngx0+2
    integer, parameter::   ig1=ig0-1
    integer, parameter::   ig2=ig0-2

    real*8,    parameter::   o12=1./12.
    real*8,    parameter::   o24=1./24.
    
   
end module
