
#source /usr/share/Modules/init/csh

module purge
module load compiler/intel/2015 mpi/intel/2015/mvapich2/2.1 netcdf/4.4.0/intel-2015_hdf5-1.8.16 pnetcdf/1.7.0/intel-2015_mvapich2-2.1
module list
export PNETCDF=/pkg/user/j07hsu00/opt/pnetcdf/1.7.0/intel-2015_mvapich2-2.1

