program preprocess
    implicit real*8 (a-h,o-z)    
    
    call init
    call run 
    call finalize
    
    stop
end program

subroutine init
    use const
    use comm
    use var


    call init_comm
    call mpi_grid2d_gen
end

subroutine run
    use const
    use comm
    use var
    use met
    use interp
    use presolver


    call metgen
    call inmets
    call indata
    call solvergen 
end

subroutine finalize()
    call mpi_finalize(ierr)
end subroutine 
